@@ -378,7 +378,7 @@ from ae.console import ConsoleApp                   # type: ignore
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
 
 
-__version__ = '0.0.18'
+__version__ = '0.0.19'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -459,8 +459,8 @@ def flow_key(flow_id: str) -> str:
     :param flow_id:         flow id string.
     :return:                flow key string.
     """
-    action_object, index = flow_key_split(flow_id)
-    return index or action_object
+    _action_object, index = flow_key_split(flow_id)
+    return index
 
 
 def flow_key_split(flow_id: str) -> Tuple[str, str]:
@@ -531,10 +531,11 @@ class MainAppBase(ConsoleApp, ABC):
 
         :param console_app_kwargs:  kwargs to be passed to the __init__ method of :class:`~ae.console_app.ConsoleApp`.
         """
-        self._exit_code: int = 0                    #: init by stop_app() and passed onto OS by run_app()
+        self._exit_code: int = 0                            #: init by stop_app() and passed onto OS by run_app()
+        self._last_focus_flow_id: str = id_of_flow('', '')  #: id of the last valid focused node item
 
-        self._start_event_loop: Optional[Callable]  #: callable to start event loop of GUI framework
-        self._stop_event_loop: Optional[Callable]   #: callable to start event loop of GUI framework
+        self._start_event_loop: Optional[Callable]          #: callable to start event loop of GUI framework
+        self._stop_event_loop: Optional[Callable]           #: callable to start event loop of GUI framework
 
         self.flow_path = list()  # init for Literal type recognition - will be overwritten by setup_app_states()
 
@@ -575,6 +576,8 @@ class MainAppBase(ConsoleApp, ABC):
         :param new_value:   new value of the app state to change.
         :param send_event:  pass False to prevent to send/call the on_<state_name> event to the main app instance.
         """
+        self.dpo(f"MainAppBase.change_app_state({state_name}, {new_value}, {send_event})"
+                 f" flow={self.flow_id} last={self._last_focus_flow_id} path={self.flow_path}")
         setattr(self, state_name, new_value)
         self._change_observable(state_name, new_value)
         if send_event:
@@ -651,6 +654,9 @@ class MainAppBase(ConsoleApp, ABC):
             if err_msg:
                 break
         self.load_cfg_files()
+
+        if self.debug_level:
+            self.play_sound('error' if err_msg else 'debug_save')
         return err_msg
 
     def setup_app_states(self, app_state: AppStateType):
@@ -661,6 +667,8 @@ class MainAppBase(ConsoleApp, ABC):
         self.dpo(f"MainAppBase.setup_app_states {app_state}")
         for key, val in app_state.items():
             self.change_app_state(key, val, send_event=False)
+            if key == 'flow_id' and flow_action(val) == 'focus':
+                self._last_focus_flow_id = val
 
         config_file_version = app_state.get(APP_STATE_VERSION_VAR_NAME, 0)
         for version in range(config_file_version, APP_STATE_CURRENT_VERSION):
@@ -731,29 +739,36 @@ class MainAppBase(ConsoleApp, ABC):
             self.dpo(f"MainAppBase.change_flow REJECTED '{ifi}'; flow='{self.flow_id}' path={self.flow_path}")
             return False
 
+        has_flow_focus = flow_action(self.flow_id) == 'focus'
         if action in ACTIONS_EXTENDING_FLOW_PATH:
             self.flow_path.append(ifi)
             self.change_app_state('flow_path', self.flow_path)
             flow_id = id_of_flow('', '') if action == 'enter' else ifi
         elif action in ACTIONS_REDUCING_FLOW_PATH:
-            flow_id = self.flow_path.pop()
+            ended_flow_id = self.flow_path.pop()
             self.change_app_state('flow_path', self.flow_path)
             if action == 'leave':
-                flow_id = replace_flow_action(flow_id, 'focus')
+                flow_id = replace_flow_action(ended_flow_id, 'focus')
+            else:
+                flow_id = self.flow_id if has_flow_focus else id_of_flow('', '')
         else:
-            flow_id = ifi
+            flow_id = ifi if action == 'focus' else (self.flow_id if has_flow_focus else id_of_flow('', ''))
 
         popups_to_close = event_kwargs.get('popups_to_close', popups_to_close)
         for widget in reversed(popups_to_close):
             widget.close()
 
-        if action not in ACTIONS_REDUCING_FLOW_PATH or flow_action(self.flow_id) != 'focus':
+        if action not in ACTIONS_REDUCING_FLOW_PATH or not has_flow_focus:
             flow_id = event_kwargs.get('flow_id', flow_id)  # update flow_id from event_kwargs set by event handler
+        if flow_id == id_of_flow('', '') and self._last_focus_flow_id:
+            flow_id = self._last_focus_flow_id
         self.change_app_state('flow_id', flow_id)
         if flow_action(flow_id) == 'focus':
             self.call_method('on_flow_widget_focused')
+            self._last_focus_flow_id = flow_id
 
-        self.dpo(f"MainAppBase.change_flow CHANGED {ifi}; flow='{self.flow_id}' path={self.flow_path}")
+        self.dpo(f"MainAppBase.change_flow CHANGED '{ifi}'; flow='{self.flow_id}' last='{self._last_focus_flow_id}'"
+                 f" path={self.flow_path}")
         return True
 
     def flow_path_action(self, path_index: int = -1) -> str:
@@ -791,9 +806,7 @@ class MainAppBase(ConsoleApp, ABC):
     # other helper methods and default event handlers
 
     def call_method(self, method: str, *args, **kwargs) -> Any:
-        """ call method of this instance with the passed args, catching exceptions.
-
-        exception catch added e.g. for ae.enaml_app (looper/focus resulting sometimes in None for item_atom kwarg).
+        """ call method of this instance with the passed args, catching and logging exceptions preventing app exit.
 
         :param method:      name of the main app method to call.
         :param args:        args passed to the main app method to be called.
@@ -841,6 +854,7 @@ class MainAppBase(ConsoleApp, ABC):
 
     def key_press_from_framework(self, modifiers: str, key: str) -> bool:
         """ dispatch key press event, coming normalized from the UI framework. """
+        self.dpo(f"MainAppBase.key_press_from_framework({modifiers}+{key})")
         event_name = f'on_key_press_of_{modifiers}_{key}'
         en_lower = event_name.lower()
         if not self.call_method(event_name):
